/************************************************************
	Alfred Lu	07/15/2007	Email:Alfred.L@hotmail.com	

	IDE:	
	MCU:	ATMega128
	FAT Interface: Support FAT16, no FAT type determination
				   no FAT volumn initialization.
				   only for file & directory operation
************************************************************/
#include <avr/io.h>         // iom128.h
#include <avr/sfr_defs.h>
#include "main.h"
#include "SD.h"
#include "FAT.h"	

UINT16  BPB_BytesPerSec;
UINT8   BPB_SecPerClus;
UINT16  BPB_RsvdSecCnt;
UINT8   BPB_NumFATs;
UINT16  BPB_RootEntCnt;
UINT16  BPB_TotSec16;
UINT16  BPB_FATSz16;
UINT32  BPB_HiddSec;

UINT32	WriteVisitedClus;  
UINT32  ReadVisitedClus;

void CopyBytes(void* S, void* D, UINT16 size)
{
	UINT8 *s = S, *d = D;
	UINT16 i;
	for(i = 0; i < size; i++)
		*d++ = *s++;
}

//
// return = equal ? 1 : 0
//
UINT8 IsEqual(void* A, void* B, UINT8 Size)
{
	UINT8 i, *a = A, *b = B;
	for(i = 0; i < Size; i++)
		if(a[i] != b[i])
			return 0;
	return 1;
}

void EmptyBytes(void* D, UINT16 size)
{
	UINT16 i;
	UINT8* data = (UINT8*)D;
	for(i = 0; i < size; i++)
	{
		*data++ = 0;
	}
}

void GetFATParameters(void)
{
	FAT_BPB BPB;
	EmptyBytes(&BPB, SIZE_OF_FAT_BPB);
	SD_ReadSingleBlock(0, &BPB);

	BPB_BytesPerSec = BPB.BPB_BytesPerSec;
	BPB_SecPerClus = BPB.BPB_SecPerClus;
	BPB_RsvdSecCnt = BPB.BPB_RsvdSecCnt;
	BPB_NumFATs = BPB.BPB_NumFATs;
	BPB_RootEntCnt = BPB.BPB_RootEntCnt;
	BPB_TotSec16 = BPB.BPB_TotSec16;
	BPB_FATSz16 = BPB.BPB_FATSz16;
	BPB_HiddSec = BPB.BPB_HiddSec;
}

//
// Get root directory start sector
//
UINT32 DirStartSec(void)
{
	return BPB_RsvdSecCnt + BPB_NumFATs * BPB_FATSz16; // reserved area + FAT table area
}

//
// Get directory sectors count
// return root directory sectors counters
//
UINT16 GetDirSecCount(void)
{
	return  (BPB_RootEntCnt * SIZE_OF_DIR + (BPB_BytesPerSec-1))/BPB_BytesPerSec;
}

//
// Get user data start sector
//
UINT32 DataStartSec(void)
{
	return (UINT32)(DirStartSec() + GetDirSecCount());
}

//
// Get Sector Num of first sector of a cluster
//
UINT32 ClusConvLBA(UINT16 ClusID)
{
	return DataStartSec() + BPB_SecPerClus * (ClusID - 2);
}

//
// Read an entry value in FAT table
//
UINT16 ReadFAT(UINT16 Index)
{
	UINT32 SecID = BPB_RsvdSecCnt + (Index >> 8);
	SD_ReadSingleBlock(SecID, BUF);
    UINT16 *BUFREF = (UINT16 *)BUF;
    return BUFREF[Index - (Index >> 8)];
}

// 
// Write an entry value in FAT table
//
void WriteFAT(UINT16 Index, UINT16 Value)
{
	UINT32 SecID = BPB_RsvdSecCnt + (Index >> 8); // get the sector ID this FAT entrancy belong to
	SD_ReadSingleBlock(SecID, BUF);
    UINT16 *BUFREF = (UINT16 *)BUF;
    BUFREF[Index - (Index >> 8)] = Value;
	SD_WriteSingleBlock(SecID, BUF);
}

//
// Get a empty entry of directory, a directory is nothing but a file contains a 
// list of 32 bytes structures
// return valide dir index
//
UINT16 GetEmptyDIR()
{
	UINT16 DirSecCut, DirStart, i, m, ID = 0;
	DirSecCut = GetDirSecCount();
	DirStart = DirStartSec();
	for(i = 0; i < DirSecCut; i++)
	{
		SD_ReadSingleBlock(DirStart + i, BUF);
		for(m = 0; m < 16; m++)
		{
			if((BUF[m<<5] == 0) || 			// this directory is free and no allocated directory entrys after this one
				(BUF[m<<5] == 0xE5))		// this directory is free
				return ID;
			else
				ID++;
		}
	}
	return ID;
}

//
// Get file dir
// return = find ? 1 : 0
// 
UINT8 GetFileDIR(void* Name, DIR* dir)
{
	UINT16 DirSecCount, DirStart, i, m;
	DirSecCount = GetDirSecCount();
	DirStart = DirStartSec();	// Get root directory start sector
	for(i = 0; i < DirSecCount; i++)
	{
		// read a single sector which contain 16 directories
		SD_ReadSingleBlock(DirStart + i, BUF);
		for(m = 0; m <16; m++)
		{
			if(IsEqual(Name, &((DIR*)&BUF[m<<5])->FileName, 11))
			{
				*dir = *((DIR*)&BUF[m<<5]);
				return 1; // find
			}
		}
	}
	return 0; // didn't find
}

// 
// Get a empty FAT entry
// 
UINT16 GetNextFAT(void)
{
	UINT16 FAT_Count, i;
	FAT_Count = BPB_FATSz16 << 8; // FAT entry counter, each entry contain 2 Bytes 
	for(i = 0; i < FAT_Count; i++)
	{
		if(ReadFAT(i) == 0)
			return i;
	}
	return 0;
}

//
// Read Directory
//
void ReadDIR(UINT16 Index, DIR* Value)
{
	UINT32 DirStart = DirStartSec();
	SD_ReadSingleBlock(DirStart + (Index>>4), BUF);
	CopyBytes(&BUF[(Index - (Index >> 4)) << 5], Value, 32);
}

//
// Write directory
//
void WriteDIR(UINT16 Index, DIR* Value)
{
	UINT32 LBA = DirStartSec() + (Index>>4);
	SD_ReadSingleBlock(LBA, BUF);
	CopyBytes(Value, &BUF[(Index - (Index >> 4)) << 5], 32);
	SD_WriteSingleBlock(LBA, BUF);
}

//
// File name must be 8.3 and be upcase
// Size calc in bytes
// Time = Hour*2048 + Min*32 + Second/2
// Data = (Year-1980)*512 + Mon*32 + Day
//
UINT8 CreateFile(void* FileName, UINT32 Size, UINT16 Time, UINT16 Data)
{
	UINT16 ClusID, ClusNum, ClusNext, i;
	DIR FileDir;
	ClusNum = Size / (BPB_SecPerClus << 9); // calc count of cluster this file needed
//  ClusNum = (Size >> 14) + 1;    // calc count of cluster this file needed
	EmptyBytes(&FileDir, sizeof(DIR));
	CopyBytes(FileName, FileDir.FileName.NAME, 11);
	FileDir.FileName.TYPE[0] = 'T';
	FileDir.FileName.TYPE[1] = 'X';
	FileDir.FileName.TYPE[2] = 'T';
	FileDir.FilePosit.Size = Size;
	FileDir.FileAttrib = ATTR_ARCHIVE;
	FileDir.FileUpdateTime[0] = Time & 0xFF;
	FileDir.FileUpdateTime[1] = Time >> 8;
	FileDir.FileUpdateData[0] = Data & 0xFF;
	FileDir.FileUpdateData[1] = Data >> 8;
	FileDir.FilePosit.Start = GetNextFAT();
	ClusID = FileDir.FilePosit.Start;
    for(i = 0; i < ClusNum; i++)
	{
	//	WriteFAT(ClusID, 0xFFFF); 
		ClusNext = GetNextFAT();
		if (ClusNext == 0)
			return 0;
		WriteFAT(ClusID, ClusNext);
		ClusID = ClusNext;
	}
	WriteFAT(ClusID, 0xFFFF);	// EOF

	WriteDIR(GetEmptyDIR(), &FileDir);

	WriteVisitedClus = 0;
	ReadVisitedClus = 0;

	CopyFAT();
	return 1;
}

void CopyFAT(void)
{
	UINT16 FATSz16, RsvdSecCnt, i;
	FATSz16 = BPB_FATSz16;
	RsvdSecCnt = BPB_RsvdSecCnt;
	for(i = 0; i < FATSz16; i++)
	{
		SD_ReadSingleBlock(RsvdSecCnt + i, BUF);
		SD_WriteSingleBlock(RsvdSecCnt + FATSz16 + i, BUF);
	}
}

//
//  ReadFile
//	Lenghth calc in bytes
//  return read bytes
//  
UINT32 ReadFile(void* Name, UINT32 Length, void*Data)
{
	UINT8 *data = Data;
	UINT16 BytePerClus, SecPerClus, ClusNum, ClusID;
	UINT32 LBA, i, j;
	DIR FileDir;
	SecPerClus = BPB_SecPerClus;
	BytePerClus = BPB_SecPerClus << 9 ; 
	if(GetFileDIR(Name, &FileDir) == 0) // haven't find file
		return 0;

	ClusNum = Length / BytePerClus;   // total cluster num for current read
//  ClusNum = Length >> 14;
	ClusID = FileDir.FilePosit.Start;
	for(i = 0; i < ReadVisitedClus; i++)
		ClusID = ReadFAT(ClusID);

	// cal offset of start position in cluster
	LBA = ClusConvLBA(ClusID);

	SD_ReadSingleBlock(LBA++, BUF);

	while(1)
	{
		for (j=0; j<SecPerClus; j++)
		{
			for (i=0; i<512; i++)
			{
				*data++ = BUF[i];
				if (--Length == 0)
				{
//					ReadVisitedClus += Length / BytePerClus;
                    ReadVisitedClus ++;
					return ((j<<9)+i+1);
				}
			}
			SD_ReadSingleBlock(LBA++, BUF);
		}
		ClusID = ReadFAT(ClusID); //Next cluster
		LBA = ClusConvLBA(ClusID);
	}
	return 0;
}

//
// WriteFile
// Length calc in bytes
// return write bytes
// 
UINT32 WriteFile(void* Name, UINT32 Length, void*Data)
{
	UINT8 *data = Data;
	UINT16 BytePerClus, SecPerClus, ClusNum, ClusID;
	UINT32 LBA, i, j;
	DIR FileDir;
	SecPerClus = BPB_SecPerClus;
	BytePerClus = BPB_SecPerClus << 9 ; 
	GetFileDIR(Name, &FileDir);

	ClusNum = Length / BytePerClus;   // total cluster num
//  ClusNum = Length >> 14;
	ClusID = FileDir.FilePosit.Start;
	for(i = 0; i < WriteVisitedClus; i++)
		ClusID = ReadFAT(ClusID);

	// cal offset of start position in cluster
	LBA = ClusConvLBA(ClusID);

	while (1)
	{
		for (j=0; j<SecPerClus; j++)
		{
			for (i=0; i<512; i++)
			{
				BUF[i] = *data++;
				if (--Length == 0)
				{
					SD_WriteSingleBlock(LBA, BUF);
//					WriteVisitedClus += Length / BytePerClus;
                    WriteVisitedClus ++;
					return ((j<<9)+i+1);
				}
			}
			SD_WriteSingleBlock(LBA++, BUF);
		}
		ClusID = ReadFAT(ClusID); //Next cluster
		LBA = ClusConvLBA(ClusID);
	}

	return 1;
}


