/************************************************************
	Alfred Lu	07/15/2007	Email:Alfred.L@hotmail.com	

	IDE:	
	MCU:	ATMega128
	FAT Interface: Support FAT16, no FAT type determination
				   no FAT volumn initialization.
				   only for file & directory operation
*************************************************************/
#ifndef _FAT_H_
#define _FAT_H_


//  BIOS parameter block (first sector in reserved region)
typedef struct		//page 34
{
	UINT8 	BS_jmpBoot[3];		// jump instruction to boot code
	UINT8 	BS_OEMName[8];		// may be "MSWIN4.1"
	UINT16 	BPB_BytesPerSec;    // bytes per sector (512 1024 2048 4096)
	UINT8 	BPB_SecPerClus;     // sector per cluster (1 2 4 8 16 32 64 128), ByterPerSec * SecPerClus < 32K
	UINT16 	BPB_RsvdSecCnt;     // reserved sector counter, must be 1
	UINT8 	BPB_NumFATs;		// number of fats, always be 2, in Flash card syetem may be 1.
	UINT16 	BPB_RootEntCnt;		// number of root directory entrancy, multipiled 32 must be even multipl of BytesPerSec
	UINT16 	BPB_TotSec16;		// total sectors (include all 4 region), must be 0 if cap larger than 10000
	UINT8 	BPB_Media;			// media ID (0xF8)
	UINT16 	BPB_FATSz16;		// sectors per one fat
	UINT16 	BPB_SecPerTrk;		// sectors per tracks for interrupt 0x13
	UINT16 	BPB_NumHeads;		// number of head for interrupt 0x13
	UINT32 	BPB_HiddSec;		// number of hidden sectors should be zero
	UINT32 	BPB_TotSec32;		// total sectors (include all 4 region)
	UINT8 	BS_DrvNum;			// int 0x13 driver number
	UINT8 	BS_Reservedl;		
	UINT8 	BS_BootSig;			// extern boot signature (0x29)
	UINT32 	BS_VolID;			// combine current data and time
	UINT8 	BS_VolLab[11];		// can be "No Name    "
	UINT8 	BS_FilSysType[8];   // "FAT     " or "FAT12   " or "FAT16   "
	UINT8 	ExecutableCode[448];// reserved
	UINT8 	ExecutableMarker[2];// 0x55 0xAA
} FAT_BPB;

#define SIZE_OF_FAT_BPB 512

#define ATTR_NORMAL		0x00
#define ATTR_READ_ONLY	0x01
#define ATTR_HIDDEN		0x02
#define ATTR_SYSTEM		0x04
#define ATTR_VOLUME_ID	0x08
#define ATTR_DIRECTORT	0x10
#define ATTR_ARCHIVE	0x20

typedef struct
{
	UINT8 NAME[8];
	UINT8 TYPE[3];
} FILE_NAME;

typedef struct
{
	UINT16 Start;
	UINT32 Size;
} FILE_POSIT;

// Directory entrancy structure
typedef struct // page 36
{
	FILE_NAME 	FileName;
	UINT8 		FileAttrib;
	UINT8 		UnUsed[10];
	UINT8 		FileUpdateTime[2];
	UINT8 		FileUpdateData[2];
	FILE_POSIT 	FilePosit;
} DIR;

#define SIZE_OF_DIR   32    // in byte

typedef struct
{
	UINT16 ClusID;
	UINT16 SecOfClus;
	UINT16 ByteOfSec;
} DATA_POSIT;


void CopyBytes(void* S, void* D, UINT16 size);
UINT8 IsEqual(void* A, void* B, UINT8 Size);
void EmptyBytes(void* D, UINT16 size);
void GetFATParameters(void);
UINT32 DirStartSec(void);
UINT32 DataStartSec(void);
UINT16 GetDirSecCount(void);
UINT32 ClusConvLBA(UINT16 ClusID);
UINT16 ReadFAT(UINT16 Index);
void WriteFAT(UINT16 Index, UINT16 Value);
UINT16 GetEmptyDIR(void);
UINT8 GetFileDIR(void* Name, DIR* dir);
UINT16 GetNextFAT(void);
void ReadDIR(UINT16 Index, DIR* Value);
void WriteDIR(UINT16 Index, DIR* Value);
UINT8 CreateFile(void* FileName, UINT32 Size, UINT16 Time, UINT16 Data);
void CopyFAT(void);
UINT32 ReadFile(void* Name, UINT32 Length, void*Data);
UINT32 WriteFile(void* Name, UINT32 Length, void*Data);
	
#endif
