/************************************************************
	Alfred Lu	07/15/2007	Email:Alfred.L@hotmail.com	

	IDE:	
	MCU:	ATMega128
	SD Interface:	SPI Mode 
************************************************************/
#ifndef _SD_H_
#define _SD_H_

// 
// Command Define (define see page 40)
// Command Format
//		b47			b46				b[45:40]		b[39:8]			b[7:1]			b0
//		0			1				x				x				x				1
//		start		transmission	cmd index		argument		CRC7			end
// 

// Command table value define
#define EMPTY	0
#define YES		1
#define NO		0
#define CMD		0
#define RD		1
#define WR		2
#define R1		0
#define R1b		1
#define R2		2
#define	R3		3

// Mask for busy token in R1b
#define 	BUSY_BIT	0x80

// Command Index
// CCC0 (basic cmd)
#define CMD0_GO_IDLE_STATE			0	// Reset all card to idle state
#define CMD1_SEND_OP_COND			1	// Asks the accessed card to send its OCR in CMD line
#define CMD9_SEND_CSD				9	// Addressed card sends its CSD on CMD
#define CMD10_SEND_CID				10	// Addressed card sends its CID on CMD
#define CMD12_STOP_TRANS			12	// Forces card to stop trans
#define CMD13_SEND_STATUS			13	// Addressed card send its status reg

// CCC2 (Block read cmd)
#define CMD16_SET_BLOCKLEN			16	// Sets blk length (in bytes) for all following block rw commands
										// Default blk length is set in CSD
#define CMD17_READ_SINGLE_BLOCK		17	// Read a block
#define CMD18_READ_MULTIPLE_BLOCK	18	// Continuous read several blocks

// CCC4 (Block write cmd)
#define CMD24_WRITE_SINGLE_BLOCK	24	// Write a block
#define CMD25_WRITE_MULTIPLE_BLOCK	25	// Continuous write several blocks
#define CMD27_PROGRAM_CSD			27	// Program CSD

// CCC5 (Erase)
#define CMD32_ERASE_WR_BLK_START	32	// Sets the address of the first write block to be erase
#define CMD33_ERASE_WR_BLK_END		33	// Sets the address of the last write block to be erase
#define CMD38_ERASE					38	// Erase all previously selected write blocks

// CCC6 (Block oriented write protection commands)

// CCC7 (Lock)

// CCC8 (ASP)
#define CMD55_APP_CMD				55	// Indicates to the card that next cmd is an application specific cmd
#define CMD56_GEN_CMD				56	// WR for general purposed(size set by CMD16)
#define CMD58_READ_OCR				58  // Read card's OCR register
#define CMD59_CRC_ON_OFF			59	// Turns CRC on or off

// ACM
#define ACMD13_SD_STATUS				13  // Send SD Memory Card Status
#define ACMD22_SEND_NUM_WR_BLOCKS		22  // Send the num of the written write blocks
#define ACMD23_SET_WR_BLK_ERASE_COUNT	23  // Set the num of write blocks to be pre-erased before writing
#define ACMD41_SD_APP_OP_COND			41  // Asks the accessed card to send its OCR in CMD line
#define ACMD42_SET_CLR_CARD_DETECT		42  // Connect/Disconnect 50KOhm pull-up resistor on CD/DAT3
#define ACMD51_SEND_SCR					51  // Reads the SCR  
		

// 
// Responds Define
//
// responds to all command except SEND_STATUS (CMD13)
#define RES_R1_IN_IDLE_STATUS		0x01	
#define RES_R1_ERASE_RESET			0x02	 
#define RES_R1_ILLEGAL_CMD			0x04
#define RES_R1_COM_CRC_ERROR		0x08
#define RES_R1_ERASE_SEQUENCE_ERROR	0x10
#define RES_R1_ADDRESS_ERROR		0x20
#define RES_R1_PARAMETER_ERROR		0x40
// responds to SEND_STATUS (CMD13)
#define RES_R2_CARD_IS_LOCKED		0x0001
#define RES_R2_LOCK_CMD_FAILED		0x0002
#define RES_R2_ERROR				0x0004
#define RES_R2_CC_ERROR				0x0008
#define RES_R2_CARD_ECC_ERROR		0x0010
#define RES_R2_WP_VIOLATION			0x0020
#define RES_R2_ERASE_PARAM			0x0040
#define RES_R2_OUT_OF_RANGE			0x0080
#define RES_R2_IN_IDLE_STATUS		0x0100
#define RES_R2_ERASE_RESET			0x0200
#define RES_R2_ILLEGAL_COMMAND		0x0400
#define RES_R2_COM_CRC_ERROR		0x0800
#define RES_R2_ERASE_SEQUENCE_ERROR	0x1000
#define RES_R2_ADDRESS_ERROR		0x2000
#define RES_R2_PARAMETER_ERROR		0x4000
// responds to data
#define RES_DATA_ACCEPTED			0x05
#define RES_DATA_CRC_ERROR			0x0B
#define RES_DATA_WRITE_ERROR		0x0D	// using CMD12 & CMD13 to stop trans and query error
// Data Error token
#define TOKEN_DATA_ERROR			0x01
#define TOKEN_DATA_CC_ERROR			0x02
#define TOKEN_DATA_ECC_FAILED		0x04
#define TOKEN_DATA_OUT_OF_RANGE		0x08		
		

// File prototype
UINT8 SD_Reset();
UINT8 SD_Send_Command(UINT8, UINT8, UINT32);
UINT8 SD_ReadSingleBlock(UINT32, void*);
UINT8 SD_WriteSingleBlock(UINT32, void*);
UINT32 SD_ReadCapacity();
UINT8 SPI_RW_Byte(UINT8 dat);

#endif
