/************************************************************
	Alfred Lu	07/15/2007	Email:Alfred.L@hotmail.com	

	IDE:	
	MCU:	ATMega128
************************************************************/
#include <avr/io.h>         // iom128.h
#include <avr/sfr_defs.h>
#include <string.h>
#include "main.h"
#include "SD.h"
#include "FAT.h"

void SPI_Set_High ()
{
	// SPI set high
	SPCR = _BV(SPE)|_BV(MSTR);	// fclk/2
	SPSR |= _BV(SPI2X);
}

void SPI_Set_Low()
{
	// set SPI low
	SPCR =  _BV(SPE)|_BV(MSTR)|_BV(SPR1)|_BV(SPR0); // fclk/128, MSB first, sample in rising edge and set in failing edge
	SPSR &= ~_BV(SPI2X);
}

void SPI_Init()
{
    // Configure port pin direction
    // PB0        PB1        PB2           PB3 
    // SS(out)    SCK(out)   MOSI(out)     MISO(in)
	DDRB = ( DDRB & ~(_BV(3))) | _BV(0) | _BV(1) | _BV(2);
    DDRA = ( DDRA | _BV(6));    // SD_CS(10/16)   

	// should not define pull up resistor
	SPI_Set_Low();

    // deassert cs signal high
    PORTB |= _BV(0);
	PORTA |= _BV(6);
}

void delay_nus(UINT16 n)
{
	while (--n);
}

//
// Use this func to read flash
//
void InitBuf()
{
	UINT16 i;
	for(i=0; i<512; i++)
	{
		DATABUF[i] = i&0xFF;
		BUF[i] = 0;
		TESTBUF[i] = 0;
	}
}

int main()
{
	UINT8 retry = 0;
	UINT8 rt = 0;
	UINT32 cap = 0;
	
	SPI_Init( ); // cs pin is deassert high
	
	// SD reset
	do 
	{
		rt = SD_Reset();
	} while(rt == 1 && retry++ < 10);
	if(retry >= 10)
		return 1;

	SPI_Set_High();

	// Read SD capacity (not necessary)
	cap = SD_ReadCapacity();

	// Get BIOS parameter
	GetFATParameters();
    
    // Prepare data buffer, this file must be rewritten
	InitBuf();

    char FILE_NAME[11];
    while(1)
    {
        // Nominate the new file, must be upcase and 8 bits
		strcpy(FILE_NAME, "ALFREDVV");

        // File size must be the multiple of 2^14 bits 
		if (CreateFile(FILE_NAME, 32768, 9849, 19767));	// size must be given when creat a new file 
		{			
            // Write Must less than or equal to 2^14 bits, 2^14 is recommended.
			rt = WriteFile(FILE_NAME, 256, DATABUF); 
			rt = WriteFile(FILE_NAME, 256, DATABUF); 
			break;
		}
	}

    return 0;
}
